@@ -788,7 +788,7 @@ BEGIN {
     my $divide_op = infix_op("/");
     my $modulo_op = infix_op("%");
     my $lshift_op = infix_op("<<");
-    my $rshift_op = infix_op("<<");
+    my $rshift_op = infix_op(">>");
     my $ncmp_op = sub { "($_[0] > $_[1] ? 1 : ($_[0] < $_[1]) ? -1 : 0)" };
     my $scmp_op = prefix_op("sv_cmp");
     my $seq_op = prefix_op("sv_eq");
@@ -1438,7 +1438,7 @@ sub compile {
 	    last OPTION;
 	} elsif ($opt eq "o") {
 	    $arg ||= shift @options;
-	    open(STDOUT, ">$arg") or return "$arg: $!\n";
+	    open(STDOUT, ">$arg") or return "open '>$arg': $!\n";
 	} elsif ($opt eq "n") {
 	    $arg ||= shift @options;
 	    $module_name = $arg;
