@@ -162,7 +162,8 @@ gv_fetchmeth(HV *stash, char *name, STRLEN len, I32 level)
     if (!stash)
 	return 0;
     if ((level > 100) || (level < -100))
-	croak("Recursive inheritance detected");
+	croak("Recursive inheritance detected while looking for method '%s' in package '%s'",
+	      name, HvNAME(stash));
 
     DEBUG_o( deb("Looking for method %s in package %s\n",name,HvNAME(stash)) );
 
@@ -1355,7 +1356,7 @@ amagic_call(SV *left, SV *right, int method, int flags)
     myop.op_next = Nullop;
     myop.op_flags = OPf_WANT_SCALAR | OPf_STACKED;
 
-    PUSHSTACK(SI_OVERLOAD);
+    PUSHSTACKi(SI_OVERLOAD);
     ENTER;
     SAVEOP();
     op = (OP *) &myop;
@@ -1380,7 +1381,7 @@ amagic_call(SV *left, SV *right, int method, int flags)
     SPAGAIN;
 
     res=POPs;
-    POPSTACK();
+    POPSTACK;
     CATCH_SET(oldcatch);
 
     if (postpr) {
