@@ -1,7 +1,7 @@
 package Text::ParseWords;
 
 use vars qw($VERSION @ISA @EXPORT $PERL_SINGLE_QUOTE);
-$VERSION = "3.1001";
+$VERSION = "3.1";
 
 require 5.000;
 
@@ -51,11 +51,7 @@ sub parse_line {
 	# We will be testing undef strings
 	local($^W) = 0;
 
-    # This is a bit of a hack, but it's the best way to work
-    # correctly most of the time.  -- Chip
-    (my $delimiter = shift) =~ s/([\s#])/\\$1/g;
-
-    my($keep, $line) = @_;
+    my($delimiter, $keep, $line) = @_;
     my($quote, $quoted, $unquoted, $delim, $word, @pieces);
 
     while (length($line)) {
@@ -80,12 +76,10 @@ sub parse_line {
 	}
         else {
 	    $unquoted =~ s/\\(.)/$1/g;
-	    if (defined $quote) {
-		$quoted =~ s/\\(.)/$1/g if ($quote eq '"');
-		$quoted =~ s/\\([\\'])/$1/g if ( $PERL_SINGLE_QUOTE && $quote eq "'");
-            }
+	    $quoted =~ s/\\(.)/$1/g if ($quote eq '"');
+	    $quoted =~ s/\\([\\'])/$1/g if ( $PERL_SINGLE_QUOTE && $quote eq "'");
 	}
-        $word .= defined $quote ? $quoted : $unquoted;
+        $word .= ($quote) ? $quoted : $unquoted;
  
         if (length($delim)) {
             push(@pieces, $word);
