@@ -1289,12 +1289,12 @@ die(const char* pat, ...)
 		msg = ERRSV;
 	    }
 
-	    PUSHSTACK(SI_DIEHOOK);
+	    PUSHSTACKi(SI_DIEHOOK);
 	    PUSHMARK(SP);
 	    XPUSHs(msg);
 	    PUTBACK;
 	    perl_call_sv((SV*)cv, G_DISCARD);
-	    POPSTACK();
+	    POPSTACK;
 	    LEAVE;
 	}
     }
@@ -1343,12 +1343,12 @@ croak(const char* pat, ...)
 	    SvREADONLY_on(msg);
 	    SAVEFREESV(msg);
 
-	    PUSHSTACK(SI_DIEHOOK);
+	    PUSHSTACKi(SI_DIEHOOK);
 	    PUSHMARK(SP);
 	    XPUSHs(msg);
 	    PUTBACK;
 	    perl_call_sv((SV*)cv, G_DISCARD);
-	    POPSTACK();
+	    POPSTACK;
 	    LEAVE;
 	}
     }
@@ -1392,12 +1392,12 @@ warn(const char* pat,...)
 	    SvREADONLY_on(msg);
 	    SAVEFREESV(msg);
 
-	    PUSHSTACK(SI_WARNHOOK);
+	    PUSHSTACKi(SI_WARNHOOK);
 	    PUSHMARK(SP);
 	    XPUSHs(msg);
 	    PUTBACK;
 	    perl_call_sv((SV*)cv, G_DISCARD);
-	    POPSTACK();
+	    POPSTACK;
 	    LEAVE;
 	    return;
 	}
@@ -2562,18 +2562,21 @@ find_script(char *scriptname, bool dosearch, char **search_ext, I32 flags)
 	if (!xfound && !seen_dot && !xfailed && (PerlLIO_stat(scriptname,&statbuf) < 0))
 #endif
 	    seen_dot = 1;			/* Disable message. */
-	if (!xfound) 
-	    scriptname = NULL;
-/*	    croak("Can't %s %s%s%s",
-		  (xfailed ? "execute" : "find"),
-		  (xfailed ? xfailed : scriptname),
-		  (xfailed ? "" : " on PATH"),
-		  (xfailed || seen_dot) ? "" : ", '.' not in PATH"); */
+	if (!xfound) {
+	    if (flags & 1) {			/* do or die? */
+	        croak("Can't %s %s%s%s",
+		      (xfailed ? "execute" : "find"),
+		      (xfailed ? xfailed : scriptname),
+		      (xfailed ? "" : " on PATH"),
+		      (xfailed || seen_dot) ? "" : ", '.' not in PATH");
+	    }
+	    scriptname = Nullch;
+	}
 	if (xfailed)
 	    Safefree(xfailed);
 	scriptname = xfound;
     }
-    return scriptname;
+    return (scriptname ? savepv(scriptname) : Nullch);
 }
 
 
